package jeliot.gui;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*; 
import javax.swing.border.*;
import javax.swing.event.*; 
import jeliot.theatre.*; 
import jeliot.parser.*;
import jeliot.*;

public class JeliotWindow {

    /** The frame in which all the action goes on. */
    private JFrame frame;

    /** The theatre in which the programs are animated. */
    private Theatre theatre;
    
    private AnimationEngine engine;
    private Jeliot jeliot;
    
    private CodePane codePane;
    private CodeEditor editor = new CodeEditor();
    
    private JSplitPane codeNest;

    private JButton stepButton;
    private JButton playButton;
    private JButton pauseButton;
    private JButton rewindButton;

    private JButton editButton;
    private JButton compileButton;

    private JTextArea outputConsole;

    private ImageLoader iLoad;
    private PanelController panelController;

    private JEditorPane errorPane = new JEditorPane(); {
        errorPane.setEditable(false);
        errorPane.setContentType("text/html");
        errorPane.setBorder(
                BorderFactory.createCompoundBorder(
                    BorderFactory.createLoweredBevelBorder(),
                    BorderFactory.createEmptyBorder(10, 10, 10, 10)
                ));
    }

    private JPanel errorViewer = new JPanel() {
        private Image backImage;
        
        public void paintComponent(Graphics g) {
            Dimension d = getSize();
            int w = d.width;
            int h = d.height;
            if (backImage == null) {
                backImage = iLoad.getLogicalImage("Panel");
            }
            int biw = backImage.getWidth(this);
            int bih = backImage.getHeight(this);
            
            if (biw >= 1 || bih >= 1) {    
                for (int x = 0; x < w; x += biw) {
                    for (int y = 0; y < h; y += bih) {
                        g.drawImage(backImage, x, y, this);
                    }
                }
            }
        }
    };
    {
        errorViewer.setBorder(
                BorderFactory.createEmptyBorder(12, 12, 5, 12));
        errorViewer.setLayout(new BorderLayout());
        errorViewer.add("Center", errorPane);
        JPanel bp = new JPanel();
        bp.setOpaque(false);
        JButton ok = new JButton("OK");
        ok.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    changeTheatrePane(theatre);
                }
            }
        );
        bp.add(ok);
        errorViewer.add("South", bp);
    }
    
    private ActionListener stepAction =
        new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                stepAnimation();
            }
        };

    private ActionListener playAction =
        new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                playAnimation();
            }
        };

    private ActionListener pauseAction =
        new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pauseAnimation();
            }
        };

    private ActionListener rewindAction =
        new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                rewindAnimation();
            }
        };

    private JFrame treeFrame = new JFrame("Program"); {
        treeFrame.setSize(300, 400);
    }
    
    private Vector editWidgets = new Vector();
    private Vector animWidgets = new Vector();

    public JeliotWindow(
            Jeliot jeliot,
            CodePane codePane,
            Theatre theatre,
            AnimationEngine engine,
            ImageLoader iLoad ) {

        this.jeliot = jeliot;
        this.codePane = codePane;
        this.theatre = theatre;
        this.engine = engine;
        this.iLoad = iLoad;

        this.panelController = new PanelController(theatre, iLoad);
    }

    public void setUp() {
        frame = new JFrame("Jeliot");
        frame.setIconImage(iLoad.getLogicalImage("Jeliot-icon"));
        
        frame.setJMenuBar(makeMenuBar());
        editor.setMasterFrame(frame);

        JSplitPane pane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
            codePane, theatre);
        pane.setOneTouchExpandable(true);
        codeNest = pane;
   
        Dimension minimumSize = new Dimension(0, 0);
        codePane.setMinimumSize(minimumSize);
        editor.setMinimumSize(minimumSize);

        JPanel bottomPane = new JPanel(new BorderLayout()) ;
            final JComponent conPan = makeControlPanel();
            bottomPane.add("West", conPan);
        
            OutputConsole oc = new OutputConsole(conPan);    
            this.outputConsole = oc;
            bottomPane.add("Center", oc.container);
        
        JPanel rootPane = new JPanel(new BorderLayout());
        
            rootPane.add("Center", pane);
        
            rootPane.add("South", bottomPane);

        frame.setContentPane(rootPane);
        frame.setSize(800, 600);
        frame.addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    System.exit(0); 
                }
            }
        );

        enterEditTrue();        
        pane.setDividerLocation(300);

        //TheatrePopup popup = new TheatrePopup();
        //theatre.addMouseListener(popup);
        //theatre.addMouseMotionListener(popup);
        
        frame.show();
    }

    private JMenuBar makeMenuBar() {
        JMenuBar menuBar = new JMenuBar();

        JMenu menu, submenu;
        JMenuItem menuItem;
        JCheckBoxMenuItem cbMenuItem;
        JRadioButtonMenuItem rbMenuItem;
        
        //a group of JMenuItems
        JMenu programMenu = editor.makeProgramMenu();
        menuBar.add(programMenu);
        editWidgets.addElement(programMenu);

        JMenu editMenu = editor.makeEditMenu();
        menuBar.add(editMenu);
        editWidgets.addElement(editMenu);

        //JMenu debugMenu = makeDebugMenu();
        //menuBar.add(debugMenu);
        
        return menuBar;
    }

    private JMenu makeDebugMenu() {
        JMenu menu = new JMenu("Debug");
        menu.setMnemonic(KeyEvent.VK_D);
        JMenuItem menuItem;

        menuItem = new JMenuItem("View Tree");
        menuItem.setMnemonic(KeyEvent.VK_T);
        menuItem.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    showProgramTree();
                }
            }
        );
        menu.add(menuItem);
        return menu;
    }

    private JButton makeControlButton(String label, String iconName) {
        ImageIcon icon = new ImageIcon("images/"+iconName);
        JButton b = new JButton(label, icon);
        b.setVerticalTextPosition(AbstractButton.BOTTOM);
        b.setHorizontalTextPosition(AbstractButton.CENTER);
    //    b.setBorder(BorderFactory.createEtchedBorder());
        b.setMargin(new Insets(0,0,0,0));
        return b;
    }

    private JPanel makeControlPanel() {
        editButton = makeControlButton("Edit", "editicon.gif");
        compileButton = makeControlButton("Compile", "compileicon.gif");

        editButton.setMargin(new Insets(0, 2, 0, 2));
        compileButton.setMargin(new Insets(0, 2, 0, 2));
        
        editButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    enterEdit();
                }
            }
        );

        compileButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    tryToEnterAnimate();
                }
            }
        );

        editWidgets.addElement(compileButton);
        animWidgets.addElement(editButton);
        
        
        JPanel statePane = new JPanel();
        statePane.setLayout(new GridLayout(1, 2));
        statePane.add(editButton);
        statePane.add(compileButton);

        // create animation control buttons
        stepButton = makeControlButton("Step", "stepicon.gif");
        playButton = makeControlButton("Play", "playicon.gif");
        pauseButton = makeControlButton("Pause", "pauseicon.gif");
        rewindButton = makeControlButton("Rewind", "rewindicon.gif");

        animWidgets.addElement(stepButton);
        animWidgets.addElement(playButton);
        animWidgets.addElement(pauseButton);
        animWidgets.addElement(rewindButton);  

        stepButton.addActionListener(stepAction);
        playButton.addActionListener(playAction);
        pauseButton.addActionListener(pauseAction);
        rewindButton.addActionListener(rewindAction);
        
        // create animation speed control slider
        final JSlider speedSlider = new JSlider(JSlider.HORIZONTAL,
                                                1, 60, 15);
        speedSlider.setMajorTickSpacing(15);
        speedSlider.setMinorTickSpacing(5);
        speedSlider.setPaintTicks(true);
        speedSlider.setPaintLabels(false);

        speedSlider.addChangeListener(
            new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    int volume = speedSlider.getValue();
                    engine.setVolume( (double)volume*100.0 );
                }
            }
        );
        animWidgets.addElement(speedSlider);

        JPanel bp = new JPanel();
        bp.setLayout(new GridLayout(1, 4));
        bp.add(stepButton);
        bp.add(playButton);
        bp.add(pauseButton);
        bp.add(rewindButton);

        JPanel p = new JPanel();
        GridBagLayout pl = new GridBagLayout();
        p.setLayout(pl);
    
        
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(0, 0, 4, 8);
        pl.setConstraints(statePane, c);
        p.add(statePane);

        JLabel jicon = new JLabel(new ImageIcon("images/jeliot.gif"));
        c.gridx = 0;
        c.gridy = 1;
        c.gridheight = 2;
        c.insets = new Insets(0, 0, 0, 0);
        pl.setConstraints(jicon, c);
        p.add(jicon);    
        
        c.gridx = 1;
        c.gridy = 0;
        c.gridheight = 1;
        c.gridwidth = 2;
        c.fill = GridBagConstraints.BOTH;
        c.insets = new Insets(0, 0, 4, 0);
        pl.setConstraints(bp, c);
        p.add(bp);
        
        c.gridy = 1;
        c.gridwidth = 1;
        c.insets = new Insets(0, 0, 0, 0);
        c.anchor = GridBagConstraints.WEST;
        JLabel label = new JLabel("Animation");
        pl.setConstraints(label, c);
        p.add(label);
        
        c.gridy = 2;
        label = new JLabel("speed");
        pl.setConstraints(label, c);
        p.add(label);

        c.gridx = 2;
        c.gridy = 1;
        c.gridheight = 2;
        pl.setConstraints(speedSlider, c);        
        p.add(speedSlider);

        p.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(5, 2, 2, 10),
                BorderFactory.createEtchedBorder()));
   
        return p;
    }


    private void enableWidgets(Enumeration enum, boolean enable) {
        while (enum.hasMoreElements()) {
            Component comp = (Component)enum.nextElement();
            comp.setEnabled(enable);
        }
    }

    private void changeCodePane(JComponent comp) {
        int loc = codeNest.getDividerLocation();
        codeNest.setLeftComponent(comp);
        codeNest.setDividerLocation(loc);
    }

    private void changeTheatrePane(JComponent comp) {
        int loc = codeNest.getDividerLocation();
        codeNest.setRightComponent(comp);
        codeNest.setDividerLocation(loc);
    }

    void enterEdit() {
        panelController.slide(false,
            new Runnable() {
                public void run() {
                    SwingUtilities.invokeLater(
                        new Runnable() {
                            public void run() {
                                enterEditTrue();
                            }
                        }
                    );
                }
            }
        ).start();
    }

    public void enterEditTrue() {
        changeCodePane(editor);
        enableWidgets(editWidgets.elements(), true);
        enableWidgets(animWidgets.elements(), false);
    }

    public String getProgram() {
        return editor.getProgram();
    }

    void showProgramTree() {
        treeFrame.setContentPane(jeliot.getTree().createPanel());
        treeFrame.setVisible(true);
    }

    void tryToEnterAnimate() {
        try{
        try {
            Reader r = new BufferedReader(
                    new StringReader(editor.getProgram()));
            changeTheatrePane(theatre);
            
            jeliot.compile(r);

            panelController.slide(true,
                new Runnable() {
                    public void run() {
                        SwingUtilities.invokeLater(
                            new Runnable() {
                                public void run() {
                                    enterAnimate();
                                }
                            }
                        );
                    }
                }
            ).start();
        }
        catch (SemanticException e) {
            showErrorMessage(e);
            return;
        }
        catch (FeatureNotImplementedException e) {
            showErrorMessage(e);
            return;
        }
        catch (SyntaxErrorException e) {
            showErrorMessage(e);
            return;
        }
        }
        catch (Exception e) {
            editButton.doClick();
            e.printStackTrace();
            System.err.println(e.getMessage());
        }
    }

    public void showErrorMessage(Exception e) {
        errorPane.setText(e.toString());
        changeTheatrePane(errorViewer);
    }

    public void showErrorMessage(SyntaxErrorException e) {
        errorPane.setText(e.toString());
        changeTheatrePane(errorViewer);
        editor.highlight(e.getLeftPos(), e.getRightPos());
    }

    public void enterAnimate() {
        changeCodePane(codePane);
        enableWidgets(editWidgets.elements(), false);
        enableWidgets(animWidgets.elements(), true);
        rewindAnimation();
    }

    void stepAnimation() {
        stepButton.setEnabled(false);
        playButton.setEnabled(false);
        pauseButton.setEnabled(true);
        rewindButton.setEnabled(false);
        editButton.setEnabled(false);

        jeliot.step();
    }

    void playAnimation() {
        stepButton.setEnabled(false);
        playButton.setEnabled(false);
        pauseButton.setEnabled(true);
        rewindButton.setEnabled(false);
        editButton.setEnabled(false);

        jeliot.play();
    }

    public void pauseAnimation() {
        stepButton.setEnabled(true);
        playButton.setEnabled(true);
        pauseButton.setEnabled(false);
        rewindButton.setEnabled(true);
        editButton.setEnabled(true);

        jeliot.pause();
    }
    
    public void resumeAnimation() {
        stepButton.setEnabled(false);
        playButton.setEnabled(false);
        pauseButton.setEnabled(true);
        rewindButton.setEnabled(false);
        editButton.setEnabled(false);
    }
    
    public void freezeAnimation() {
        stepButton.setEnabled(false);
        playButton.setEnabled(false);
        pauseButton.setEnabled(false);
        rewindButton.setEnabled(false);
        editButton.setEnabled(true);
    }

    void rewindAnimation() {
        stepButton.setEnabled(true);
        playButton.setEnabled(true);
        pauseButton.setEnabled(false);
        rewindButton.setEnabled(false);

        jeliot.rewind();
        theatre.repaint();
    }

    public void animationFinished() {
        stepButton.setEnabled(false);
        playButton.setEnabled(false);
        pauseButton.setEnabled(false);
        rewindButton.setEnabled(true);
        editButton.setEnabled(true);     
    }

    public void output(String str) {
        outputConsole.append(str);
    }
}
