package jeliot.parser;

import java.util.*;
import jeliot.lang.*;
import jeliot.theatre.*;
import jeliot.parser.error.*;
import jeliot.FeatureNotImplementedException;

/**
*
* @author Pekka Uronen
*
* created         24.8.1999
*/
public class FieldAccess extends PVariableAccessExpr {
    
    FieldAccess access;
    FieldPointer field;
 
    public FieldAccess(PName name) {
        super(name);
    }
    
    public Type resolveType(
            NameSpace space,
            LocalVariableBundle locals,
            PClass context) {
                
        try { 
            PName name = getName();
            if (name.isQualified()) {
                // this field access is of form lext.head
                PName left = name.getLeftHand();
                String head = name.getHead();
                if (left.getClassification() == PName.TYPE) {
                    // left hand side is a class name
                    ReferenceType rt = space.findClass(left.toString());
                    
                    // set the referenced field  
                    this.field = rt.getField(head);
                    
                    // set the type of this expression as the type of
                    // the referenced field.
                    Type type = field.getType();
                    setType(type);
                    return type;
                }
                // left hand side is not a class name, so treat it as
                // a field access.
                this.access = new FieldAccess(left);
                access.setParent(this);
                
                // get the type of the access
                Type pt = access.resolveType(space, locals, context);
                if (! (pt instanceof ReferenceType)) {
                    throw new SemanticException("BloBLob");    
                }
                ReferenceType rt = (ReferenceType)pt;
                
                // get the referenced field
                this.field = rt.getField(head);
                
                // set the type of this expression as the type of
                // the referenced field.                    
                Type type = field.getType();
                setType(type);
                return type;                              
            }
            // simple name
            
            // get the name of the referenced field
            String id = name.getHead();
            
            //  Check if the access is to a local variable
            PVariableDeclarator vacc = 
                    (PVariableDeclarator)locals.get(id);
            if (vacc != null) {   
                // refering to a local variable
                
                // get the referenced variable
                this.field = vacc.getPointer();
                
                // set the type of this expression as the type of
                // the referenced variable.
                Type type = field.getType();
                setType(type);
                return type;
            }
            // not a local variable, so search the fields of 
            // enclosing class
            vacc = context.getField(id);
            if (vacc != null) {
                field = vacc.getPointer();
                Type type = field.getType();
                setType(type);
                return type;
            }
            throw new NonexistentFieldException(context.getType(), id);
        }
        catch (NonexistentFieldException e) {
            e.setAccess(this);
            throw (e);
        }       
    }
    
    public void acceptVisitor(ProgramTreeVisitor visitor) {
        if (access != null) {
            access.acceptVisitor(visitor);
        }
        super.acceptVisitor(visitor);
    }
    
}
