package jeliot.parser;

import jeliot.lang.*;
import jeliot.theatre.*;
import jeliot.FeatureNotImplementedException;

/**
  * PAssignmentExpr represents an assignment expression.
  *
  * @author Pekka Uronen
  *
  * created         6.8.1999
  * last modified   16.8.1999
  */
public class PAssignmentExpr extends PExpression {

    public static final int NOP     = -1;
    
    protected int binaryType;
    protected PExpression leftHand;
    protected PExpression rightHand;
   
    public PAssignmentExpr() {
        this.binaryType = NOP;
    }

    public PAssignmentExpr(int binaryType) {
        this.binaryType = binaryType;
        throw new FeatureNotImplementedException(
                "Compound assignment");
    }

    public void setSides(PExpression leftHand, PExpression rightHand) {
        this.leftHand = leftHand;
        this.rightHand = rightHand;

        leftHand.setParent(this);
        rightHand.setParent(this);
        
        setLeftPos(leftHand.getLeftPos());
        setRightPos(rightHand.getRightPos());
        
    }

    public PExpression getLeftHand() {
        return leftHand;
    }

    public PExpression getRightHand() {
        return rightHand;
    }

    public void acceptVisitor(ProgramTreeVisitor visitor) {
        visitor.enterAssignmentExpression(this);
        leftHand.acceptVisitor(visitor);
        rightHand.acceptVisitor(visitor);
        visitor.leaveAssignmentExpression(this);
    }

    public Value evaluate(Director director) {
        super.evaluate(director);
        Variable var = leftHand.evaluateVariable(director);
        Value val = rightHand.evaluate(director);
        var.assign(val);
        Value casted = var.getValue();
        Value retval = (Value)casted.clone();
        director.atExpression(this);
        director.animateAssignment(var, val, casted, retval);
        return retval;
    }
    
    public boolean usesScratch() {
        return false;
    }
}
